# Python Class for controlling ble based light bulbs
import pexpect
import time
from time import gmtime, strftime
from iota_command_maker import command_color_gen, command_gen



class Bulb:
	"Common base class for all the bulbs"

	def __init__(self, mac_address):
		self.child = pexpect.spawn("gatttool -I")
		self.mac_address = mac_address
	
	def connect(self):
		self.child.sendline("connect {0}".format(self.mac_address))
		self.child.expect("Connection successful", timeout=5)
		#return self.child

	def write(self, handle, command):
		full_command = "char-write-req %s %s" % (handle,command)
		self.child.sendline(full_command)
		self.child.expect("Characteristic value was written successfully", timeout=5)


class IotaBulb(Bulb):
	"Inherited class for Iota Light bulb"

	def __init__(self, mac_address, handle='0x002b'):
		super().__init__(mac_address)
		self.handle = handle


	def on(self, color='ffffff'):
		"""Turns the bulb on, accept color in ffffff format: bulb.on('ffffff') """
		command = command_gen('on', color)
		self.write(self.handle, command)

	def off(self):
		""" Turns the bulb off """
		command = command_gen('off')
		self.write(self.handle, command)

	def color(self, color, brightness=100):
		""" Change the color and brightness, brightness is optional: bulb.color('ffffff', 50)
		Note: This bulb only support brightness values from 8 to 100  """
		if brightness < 8:
			brightness = 8
			# print('This bulb only support brightness value greater than 7')
		command = command_color_gen(color, brightness)
		self.write(self.handle, command)


import re


def checksum_gen(packet):
	packetsum = 0
	for i in re.findall('..', packet):
		packetsum += int(i, 16)

	p2 = packetsum + 0xE8

	checksum = (packetsum + 0xE8) & 0xFF
	return (hex(checksum ))

def checksum_gen_color(packet):
	packetsum = 0

	for i in re.findall('..', packet):
		packetsum += int(i, 16)

	p2 = packetsum + 0xE5
	checksum = (packetsum + 0xE5) & 0xFF
	return (hex(checksum ))


def command_gen(operation, color='000000'):
	header = '0x0f0a0d00'
	color = '0x' + color
	on_command = '0x00050000'
	off_command = '0xc8050000'

	if operation == "off":
		mydata = [header, color, on_command]
	elif operation == "on":
		mydata = [header, color, off_command]
	else:
		print('unknown command')
	
	packet = ''.join([i[2:] for i in mydata])[:-2]
	checksum = checksum_gen(packet)
	final_packet = ''.join([i[2:] for i in mydata])
	final_command = final_packet + checksum[2:] + 'ffff'
	return final_command


def command_color_gen(color, brightness):
	color = '0x' + color
	brightness = hex(brightness*2)
	header = '0x0f0d0300'
	#color = '0x000000'
	#brightness = '0xc8'
	padding = '0x000000000000'

	mydata = [header, color, brightness, padding, ]
	
	packet = ''.join([i[2:] for i in mydata])[:-2]
	checksum = checksum_gen_color(packet)
	final_packet = ''.join([i[2:] for i in mydata])
	final_command = final_packet + checksum[2:] + 'ffff'
	return final_command
